import unittest
import AutoRPE.UtilsRPE.Getter as Getter
import AutoRPE.UtilsRPE.VariablePrecision as VariablePrecision


class TestGetter(unittest.TestCase):

    def test_get_function_type(self):
        from AutoRPE.UtilsRPE.Classes.Vault import Vault
        import AutoRPE.UtilsRPE.Classes.BasicStructures as BasicStructures
        subroutine = BasicStructures.SubRoutine("dia_hth_dep", None)
        var_1 = BasicStructures.Variable(subroutine, "Kmm", _type="integer", _dimension=1, _module=None)
        subroutine.accessible_var["Kmm"] = [var_1]
        vault = Vault()
        string = "LOG(Kmm)"
        self.assertEqual(Getter.get_function_type(string, subroutine, vault), 'integer')
        string = "INT(Kmm)"
        self.assertEqual(Getter.get_function_type(string, subroutine, vault), 'integer')
        string = "cmplx(Kmm)"
        self.assertEqual(Getter.get_function_type(string, subroutine, vault), 'complex')
        string = "trim(Kmm)"
        self.assertEqual(Getter.get_function_type(string, subroutine, vault), 'char')
        string = " present(Kmm)"
        self.assertEqual(Getter.get_function_type(string, subroutine, vault), 'logical')
        string = "c_loc(Kmm)"
        self.assertEqual(Getter.get_function_type(string, subroutine, vault), 'pointer')
        string = "rpe(Kmm)"
        self.assertEqual(Getter.get_function_type(string, subroutine, vault), 'rpe_var')
        string = "float(Kmm)"
        self.assertEqual(Getter.get_function_type(string, subroutine, vault), VariablePrecision.real_id['wp'])

    def test_type_of_real_cast_with_double_specification(self):
        string = 'real((ssh(:,:,kmm)+ssh_ref)*tmask(:,:,1),dp)'
        self.assertEqual(Getter.get_type_of_real_cast(string), 'dp')

    def test_type_of_real_cast_with_single_specification(self):
        string = 'real((ssh(:,:,kmm)+ssh_ref)*tmask(:,:,1),sp)'
        self.assertEqual(Getter.get_type_of_real_cast(string), VariablePrecision.real_id['sp'])

    def test_type_of_real_cast_no_specification(self):
        string = 'real((ssh(:,:,kmm)+ssh_ref)*tmask(:,:,1))'
        self.assertEqual(Getter.get_type_of_real_cast(string), VariablePrecision.real_id['wp'])

    def test_set_working_precision(self):
        VariablePrecision.set_working_precision("sp")
        self.assertEqual(VariablePrecision.real_id['wp'], VariablePrecision.real_id['sp'])
        VariablePrecision.set_working_precision("dp")
        self.assertEqual(VariablePrecision.real_id['wp'], VariablePrecision.real_id['dp'])

    def test_dimension_of_contents_1(self):
        self.assertEqual(Getter.get_dimension_of_contents("1.0"), 0)

    def test_dimension_of_contents_2(self):
        string = "real(  ( ( z3d(:,:,:) - e3t_0(:,:,:) ) / e3t_0(:,:,:) * 100 * tmask(:,:,:) ) ** 2  )"
        dimension = Getter.get_dimension_of_contents(string)
        self.assertEqual(dimension, 3)

    def test_dimension_of_contents_3(self):
        string = "e1e2t(:,:) * zfwf(:,:)"
        dimension = Getter.get_dimension_of_contents(string)
        self.assertEqual(dimension, 2)

    def test_dimension_of_contents(self):
        cases = [
            ('SUM( a_i(:,:,:)%val - a_i_b(:,:,:)%val, dim=3 )', 2),
            ('SQRT( ( zs1(:,:)%val - zs2(:,:)%val )**2 + 4*zs12(:,:)%val**2 ) * zmsk00(:,:)', 2)
        ]
        for case, answer in cases:
            self.assertEqual(Getter.get_dimension_of_contents(case), answer)

    # Simple Array
    def test_clean_remove_indexing_1(self):
        string = 'uu(:,:,:,Kmm)'
        result = Getter.remove_indexing(string)
        expected_result = "uu"
        self.assertEqual(result, expected_result)

    # Array with array indexing
    def test_clean_remove_indexing_2(self):
        string = 'zz_read(1:nblendta(igrd,ib_bdy),:)'
        result = Getter.remove_indexing(string)
        expected_result = "zz_read"
        self.assertEqual(result, expected_result)

    # Structure with array member
    def test_clean_remove_indexing_3(self):
        string = 'idx_bdy(ib_bdy)%ntreat(ib,igrd)'
        result = Getter.remove_indexing(string)
        expected_result = 'idx_bdy%ntreat'
        self.assertEqual(result, expected_result)

    # Structure with structure indexing
    def test_clean_remove_indexing_4(self):
        string = 'sdjf%nrecsec(1:sdjf%nreclast)'
        result = Getter.remove_indexing(string)
        expected_result = "sdjf%nrecsec"
        self.assertEqual(result, expected_result)


if __name__ == '__main__':
    unittest.main()
